/*
 *  libsocket - BSD socket like library for DJGPP
 *  Copyright 1997, 1998 by Indrek Mandre
 *  Copyright 1997, 1998 by Richard Dawe
 *
 *  This library is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 * 
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Original code from Dan Hedlund's wsock library, modified by Indrek Mandre
 * (IM) and Richard Dawe (RD).
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/farptr.h>
#include <sys/segments.h>
#include <dpmi.h>
#include <signal.h>
#include <pc.h>
#include <errno.h>

#include "vxd.h"
#include "glob.h"
#include "ws.h"

int WSockEntry [2];
int WSockEntry2 [2];    /* RD: Only used to check if Winsock 2 is present */

/* Used to allocate DPMI memory */
__dpmi_meminfo _SocketP;
__dpmi_meminfo _SocketD;
/*__dpmi_meminfo _SocketC;*/

/* RD: DPMI selectors */
int SocketP;    /* Winsock parameters */
int SocketD;    /* Winsock data (used as well as param's in some func's */
/*int SocketC; */   /* Code - set to own program's area  */

int CallBack;

/*int WSockDataSel = 0;*/   /* RD: Not used anywhere else! */

extern char WS_DNS[256];
extern int WS_DNS_inited;

static int WS_initialized = 0;  /* RD: Moved here from socket.c */

void WS_close2 ( int );

/* RD: WS_init now returns 1 on success, 0 on failure. If it's previously
   init'd OK, then it returns 1. */

int WS_init (void)
{    
    int i;

    /* RD: Done this before, so it's OK */
    if ( WS_initialized ) return(1); 

    if ( ! WS_DNS_inited ) {
         WS_DNS[0] = 0;
        WS_DNS_inited = 1;
    }

    /* RD: memset code moved from end of WS_init() - safer to do it here */
    /*memset ( WSdescr, 0, 256 );*/ /* RD: for loop better, I think */

    for (i = 0; i < 256; i++) WSdescr[i] = NULL;    

    /* RD: Load Winsock VxDs - try loading Winsock 2 to see if we have
           Winsock 2 */
    VxdLdrLoadDevice ("WSOCK.VXD");
    VxdLdrLoadDevice ("WSOCK.386");
    VxdLdrLoadDevice ("WSOCK2.VXD");

    VxdGetEntry (WSockEntry, 0x003E);
    VxdGetEntry (WSockEntry2, 0x003A);

    /* RD: Check the WSOCK.VXD entry point is valid - 0:0 = invalid, others OK */
    if ( (WSockEntry[0] == 0) && (WSockEntry[1] == 0) ) {
        errno = ENODEV; /* "No such device" */
        return(0);
    }

    /* RD: If the WSOCK2.VXD entry point is valid then return, as the code
           currently doesn't work with Winsock 2's WSOCK.VXD */
    if ( (WSockEntry2[0] != 0) || (WSockEntry2[1] != 0) ) {
        errno = ENOSYS;
        return(0);
    }

    _SocketP.handle = 0;
    _SocketP.size = 4096;                   /* RD: 65536 is big */
    _SocketP.address = 0;
    __dpmi_allocate_memory (&_SocketP);

    _SocketD.handle = 0;
    _SocketD.size = 4096;                   /* RD: 65536 is big */
    _SocketD.address = 0;
    __dpmi_allocate_memory (&_SocketD);

    /*_SocketC.handle = 0;
    _SocketC.size = 65536;
    _SocketC.address = 0;
    __dpmi_allocate_memory (&_SocketC);*/

    SocketP = __dpmi_allocate_ldt_descriptors (1);
    SocketD = __dpmi_allocate_ldt_descriptors (1);
    /*SocketC = __dpmi_allocate_ldt_descriptors (1);*/

    __dpmi_set_segment_base_address (SocketP, _SocketP.address);
    __dpmi_set_segment_base_address (SocketD, _SocketD.address);
    /*__dpmi_set_segment_base_address (SocketC, _SocketC.address);*/

    __dpmi_set_segment_limit (SocketP, _SocketP.size);
    __dpmi_set_segment_limit (SocketD, _SocketD.size);
    /*__dpmi_set_segment_limit (SocketC, _SocketC.size);*/

    /* RD's guessed purpose for Dan Hedlund's "SocketC" code: I think code is
       here to allow Winsock to call back the program via the address of a
       function stored in CallBack. Currently none of the code seems to use
       this, but it might be possible to asynchronous operations with it. */

    /* RD: Does this code perform a callback? i.e. it calls the program's
       specified callback procedure when called by Winsock? */

    /*_farpokel (SocketC, 0, 0xea676690);
    _farpokel (SocketC, 4, (int) (&CallBack));
    _farpokel (SocketC, 8, _my_cs());*/

    /* Set descriptor access rights: Non-conforming code, readable, accessed */
    /*__dpmi_set_descriptor_access_rights (SocketC, 0x00fb);*/

    /*WSockDataSel = _my_ds ();*/   /* RD: Not used anymore */

/*   signal ( SIGINT, WS_close2 );
    signal ( SIGSEGV, WS_close2 );
    signal ( SIGABRT, WS_close2 );
      IM: bad idea I think */

    /* RD: Set this now we've done everything successfully */
    WS_initialized = 1;

    /* RD: This is better last */
    atexit(WS_close);

    /* Success */
    return(1);
}

/* RD: WS_close[2] takes an int - Why? */

void WS_close2 ( int v )
{
    int i;

    if ( !WS_initialized ) return;
    WS_initialized = 0;

    /* RD: Check not NULL, not just set. The memory is free'd by closing the
       socket */
    for (i = 0; i < 256; i++) { if (WSdescr[i] != NULL) close(i); }

    /* Free DPMI memory */
    /*__dpmi_free_memory(_SocketP.handle);
    __dpmi_free_memory(_SocketD.handle);
    __dpmi_free_memory(_SocketC.handle);*/

    /* Free descriptors */
    /*__dpmi_free_ldt_descriptor(SocketP);
    __dpmi_free_ldt_descriptor(SocketD);
    __dpmi_free_ldt_descriptor(SocketC);*/

    VxdLdrUnLoadDevice ("WSOCK.VXD");
    VxdLdrUnLoadDevice ("WSOCK.386");
    /* RD: Winsock 2 too! */
    VxdLdrUnLoadDevice ("WSOCK2.VXD");
}

void WS_close (  )
{
    int i;

    if (!WS_initialized) return;
    WS_initialized = 0;

    /* RD: Check not NULL, not just set. The memory is free'd by closing the
       socket */
    for (i = 0; i < 256; i++) { if (WSdescr[i] != NULL) close(i); }

    /* Free DPMI memory */
    __dpmi_free_memory(_SocketP.handle);
    __dpmi_free_memory(_SocketD.handle);
    /*__dpmi_free_memory(_SocketC.handle);*/

    /* Free descriptors */
    __dpmi_free_ldt_descriptor(SocketP);
    __dpmi_free_ldt_descriptor(SocketD);
    /*__dpmi_free_ldt_descriptor(SocketC);*/

    /* IM: let's keep these unchanged, they don't hurt... */
    /* VxdLdrUnLoadDevice ("WSOCK.VXD");
       VxdLdrUnLoadDevice ("WSOCK.386"); */
    /* RD: Winsock 2 too! */
    /* VxdLdrUnLoadDevice ("WSOCK2.VXD"); */
}

